\\ Copyright 2019, 2023 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\-----------------------------------------------------------------------------
\\ with own state watching code, rather than using vecper_make_state()

vecper_apply_state(func,v,state=0) =
{
  \\ print("vecper_apply_state()");
  my(ret);
  for(i=1,#v[1],
     ret=func(v[1][i],state);
     v[1][i]=ret[1];
     state  =ret[2]);
  my(seen=List([state]), seen_at=List([0]), l=List([]));
  while(1,
        \\ print("  seen "Vec(seen));
        for(i=1,#v[2],
           ret=func(v[2][i],state);
           listput(l,ret[1]);
           state=ret[2]);
        my(pos=setsearch(seen,state));
        if(pos,
           \\ print("  repeat state "state" pos="pos" at "seen_at[pos]);
           \\ print("  l="Vec(l));
           if(seen_at[pos],
              v[1]=concat(v[1],Vec(l,seen_at[pos]));
              v[2]=Vec(l)[seen_at[pos]+1..#l];
              ,
              v[2]=Vec(l));
           break);
        pos=setsearch(seen,state,1);
        listinsert(seen,state,pos);
        listinsert(seen_at,#l,pos));
  v;
}


\\-----------------------------------------------------------------------------

\\ x is a floating point (t_REAL), return it as a rational (t_FRAC)
float_to_rat(x) =
{
  if(x==0, 0,
     x==1, 1,
     \\ binary() of a float gives say [ [0], [1,0,1,0,...] ]
     \\ which is bits above the binary point then bits below
     my(b=binary(x));
     (#b == 2 && type(b[1])=="t_VEC")
       || error("oops, not float style from binary()");
     fromdigits(b[1],2) + fromdigits(b[2],2)/2^#b[2]);
}
float_to_rat(.25) == 1/4  || error();


\\-----------------------------------------------------------------------------

\\ v is a vector.
\\ Return a "rotated up" [ v[#v], v[1], v[2], ..., v[#v-1] ].
\\ n is how many places to rotate this way.
\\ If v is less than 2 elements then return unchanged.
vecper_INTERNAL_vector_rotate_up(v,n=1) = \
  if(#v<2 || !(n%=#v), v, n=#v-n; concat([v[n+1..#v], v[1..n]]));

\\---------
\\ vecper_INTERNAL_vector_rotate_up()

check_equal(vecper_INTERNAL_vector_rotate_up([]), [], "up []");
check_equal(vecper_INTERNAL_vector_rotate_up([1]), [1], "up [1]");
check_equal(vecper_INTERNAL_vector_rotate_up([10,20]), [20,10], "up [10,20]");
check_equal(vecper_INTERNAL_vector_rotate_up([10,20,30]), [30,10,20], "up [10,20,30]");
check_equal(vecper_INTERNAL_vector_rotate_up([10,20,30],1), [30,10,20]);
check_equal(vecper_INTERNAL_vector_rotate_up([10,20,30],2), [20,30,10]);
check_equal(vecper_INTERNAL_vector_rotate_up([10,20,30],3), [10,20,30]);

check_equal(vecper_INTERNAL_vector_rotate_up([10,20,30,40],0), [10,20,30,40], \
            "vecper_INTERNAL_vector_rotate_up()");
check_equal(vecper_INTERNAL_vector_rotate_up([10,20,30,40],1), [40,10,20,30], \
            "vecper_INTERNAL_vector_rotate_up()");

\\-----------------------------------------------------------------------------
